/*
   ThreeServos.cpp

    Shows smooth movement from one servo position to another for 3 servos synchronously.

    Copyright (C) 2019  Armin Joachimsmeyer
    armin.joachimsmeyer@gmail.com

    This file is part of ServoEasing https://github.com/ArminJo/ServoEasing.

    ServoEasing is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/gpl.html>.
*/

#include <Arduino.h>
#include "Arduino.h"
#include "SoftwareSerial.h"
#include "DFRobotDFPlayerMini.h"
#include "ServoEasing.h"

#define VERSION_EXAMPLE "1.4"
#define LED_BUILTIN PB1
const int SERVO1_PIN = 9;
const int SERVO2_PIN = 10;
const int SERVO3_PIN = 11;
const int SERVO4_PIN = 12;
ServoEasing Servo1;
ServoEasing Servo2;
ServoEasing Servo3;
ServoEasing Servo4;
SoftwareSerial mySoftwareSerial(3, 2); // RX, TX
DFRobotDFPlayerMini myDFPlayer;
void printDetail(uint8_t type, int value);
//void blinkLED();

void setup() {
  mySoftwareSerial.begin(9600);
  Serial.begin(115200);
  Serial.println();
  Serial.println(F("DFRobot DFPlayer Mini Demo"));
  Serial.println(F("Initializing DFPlayer ... (May take 3~5 seconds)"));
  if (!myDFPlayer.begin(mySoftwareSerial)) {  //Use softwareSerial to communicate with mp3.
    Serial.println(F("Unable to begin:"));
    Serial.println(F("1.Please recheck the connection!"));
    Serial.println(F("2.Please insert the SD card!"));
    while (true);
  }
  Serial.println(F("DFPlayer Mini online."));
  myDFPlayer.volume(30);  //Set volume value. From 0 to 30
  myDFPlayer.play(1);  //Play the first mp3
  Serial.println();
  Serial.println(F("DFRobot DFPlayer Mini Demo"));
  pinMode(LED_BUILTIN, OUTPUT);

  // Attach servos to pins
#ifndef PRINT_FOR_SERIAL_PLOTTER
  Serial.print(F("Attach servo at pin "));
  Serial.println(SERVO1_PIN);
#endif
  if (Servo1.attach(SERVO1_PIN) == INVALID_SERVO) {
    Serial.println(F("Error attaching servo"));
  }

#ifndef PRINT_FOR_SERIAL_PLOTTER
  Serial.print(F("Attach servo at pin "));
  Serial.println(SERVO2_PIN);
#endif
  if (Servo2.attach(SERVO2_PIN) == INVALID_SERVO) {
    Serial.println(F("Error attaching servo"));
  }
#ifndef PRINT_FOR_SERIAL_PLOTTER
  Serial.print(F("Attach servo at pin "));
  Serial.println(SERVO4_PIN);
#endif
  if (Servo4.attach(SERVO4_PIN) == INVALID_SERVO) {
    Serial.println(F("Error attaching servo"));
  }
  /*
     Check at least the last call to attach()
  */
#ifndef PRINT_FOR_SERIAL_PLOTTER
  Serial.print(F("Attach servo at pin "));
  Serial.println(SERVO3_PIN);
#endif
  if (Servo3.attach(SERVO3_PIN) == INVALID_SERVO) {
    Serial.print(F("Error attaching servo - maybe MAX_EASING_SERVOS="));
    Serial.print(MAX_EASING_SERVOS);
    Serial.println(F(" is to small to hold all servos"));
    while (true) {
      blinkLED();
    }
  }

  Servo1.print(&Serial);
  Servo2.print(&Serial);
  Servo3.print(&Serial);
  Servo4.print(&Serial);
  /**************************************************
     Set servos to start position.
     This is the position where the movement starts.
   *************************************************/
  Servo1.write(0);
  Servo2.write(0);
  Servo3.write(0);
  Servo4.write(0);
  // Legend for Arduino plotter
  //Serial.println("Servo 1, Servo 2, Servo 3");

  // Wait for servos to reach start position.
  delay(500);
}

void blinkLED() {
  digitalWrite(LED_BUILTIN, HIGH);
  delay(100);
  digitalWrite(LED_BUILTIN, LOW);
  delay(100);
}

void loop() {
  Serial.println("--Generate values--");
  /*
     Move three servos synchronously without interrupt handler
  */
#ifndef PRINT_FOR_SERIAL_PLOTTER
  //Serial.println(F("Move to 90/90/180 degree with 20 degree per second with own update loop"));
#endif
  int randomS = 150;
  //Serial.println(randomS);
  //setSpeedForAllServos(150);
  setSpeedForAllServos(randomS);
  int random1 = random(10, 45);
  int random2 = random(0, 180);
  int random3 = random(0, 180);
  int random4 = random(110, 180);
  //delay(100);
  synchronizeAllServosAndStartInterrupt(false); // do not start interrupt

  do {

    Serial.println("Servo 1 moving to value:");
    Serial.println(random1);
    Serial.println("Servo 2 moving to value:");
    Serial.println(random2);
    Serial.println("Servo 3 moving to value:");
    Serial.println(random3);
    Serial.println("Servo 4 moving to value:");
    Serial.println(random4);


    Servo1.setEaseTo(random1);
    delay(1);
    Servo2.setEaseTo(random2);
    delay(1);
    Servo3.setEaseTo(random3);

    Servo4.setEaseTo(random4);
    // here you can call your own program
    delay(REFRESH_INTERVAL / 1000); // optional 20ms delay - REFRESH_INTERVAL is in Microseconds
  } while (!updateAllServos());
  delay(200);
  Serial.println("--end--");

  static unsigned long timer = millis();

  if (millis() - timer > 3000) {
    timer = millis();
    myDFPlayer.next();  //Play next mp3 every 3 second.
  }

  if (myDFPlayer.available()) {
    printDetail(myDFPlayer.readType(), myDFPlayer.read()); //Print the detail message from DFPlayer to handle different errors and states.
  }

}


void printDetail(uint8_t type, int value) {
  switch (type) {
    case TimeOut:
      Serial.println(F("Time Out!"));
      break;
    case WrongStack:
      Serial.println(F("Stack Wrong!"));
      break;
    case DFPlayerCardInserted:
      Serial.println(F("Card Inserted!"));
      break;
    case DFPlayerCardRemoved:
      Serial.println(F("Card Removed!"));
      break;
    case DFPlayerCardOnline:
      Serial.println(F("Card Online!"));
      break;
    case DFPlayerPlayFinished:
      Serial.print(F("Number:"));
      Serial.print(value);
      Serial.println(F(" Play Finished!"));
      break;
    case DFPlayerError:
      Serial.print(F("DFPlayerError:"));
      switch (value) {
        case Busy:
          Serial.println(F("Card not found"));
          break;
        case Sleeping:
          Serial.println(F("Sleeping"));
          break;
        case SerialWrongStack:
          Serial.println(F("Get Wrong Stack"));
          break;
        case CheckSumNotMatch:
          Serial.println(F("Check Sum Not Match"));
          break;
        case FileIndexOut:
          Serial.println(F("File Index Out of Bound"));
          break;
        case FileMismatch:
          Serial.println(F("Cannot Find File"));
          break;
        case Advertise:
          Serial.println(F("In Advertise"));
          break;
        default:
          break;
      }
      break;
    default:
      break;
  }

}
